@@ -3,7 +3,7 @@
  *
  * Create a repository schema.
  *
- * Copyright (c) 2009-2017, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+ * Copyright (c) 2009-2018, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
  */
 
 -- Adjust this setting to control where the objects get created.
@@ -533,11 +533,87 @@ CREATE TABLE statsrepo.replication
 	write_location		text,
 	flush_location		text,
 	replay_location		text,
+	write_lag			interval,
+	flush_lag			interval,
+	replay_lag			interval,
 	sync_priority		integer,
 	sync_state			text,
 	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
 );
 
+CREATE TABLE statsrepo.replication_slots
+(
+	snapid					bigint,
+	slot_name				name,
+	plugin					name,
+	slot_type				text,
+	datoid					oid,
+	temporary				boolean,
+	active					boolean,
+	active_pid				integer,
+	xact_xmin				xid,
+	catalog_xmin			xid,
+	restart_lsn				pg_lsn,
+	confirmed_flush_lsn		pg_lsn,
+	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
+);
+
+CREATE TABLE statsrepo.publication
+(
+	snapid			bigint,
+	dbid			oid,
+	pubid			oid,
+	pubname			name,
+	pubowner		oid,
+	puballtables	boolean,
+	pubinsert		boolean,
+	pubupdate		boolean,
+	pubdelete		boolean,
+	PRIMARY KEY (snapid, dbid, pubid),
+	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
+);
+
+CREATE TABLE statsrepo.publication_tables
+(
+	snapid			bigint,
+	dbid			oid,
+	pubname			name,
+	schemaname		name,
+	tablename		name,
+	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
+);
+
+CREATE TABLE statsrepo.subscription
+(
+	snapid					bigint,
+	subid					oid,
+	subdbid					oid,
+	subname					name,
+	subowner				oid,
+	subenabled				boolean,
+	subconninfo				text,
+	subslotname				name,
+	subsynccommit			text,
+	subpublications			text[],
+	PRIMARY KEY (snapid, subid),
+	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
+);
+
+CREATE TABLE statsrepo.stat_subscription
+(
+	snapid					bigint,
+	subid					oid,
+	subname					name,
+	pid						integer,
+	relid					oid,
+	received_lsn			pg_lsn,
+	last_msg_send_time		timestamptz,
+	last_msg_receipt_time	timestamptz,
+	latest_end_lsn			pg_lsn,
+	latest_end_time			timestamptz,
+	FOREIGN KEY (snapid) REFERENCES statsrepo.snapshot (snapid) ON DELETE CASCADE
+);
+
 CREATE TABLE statsrepo.xlog
 (
 	snapid		bigint,
@@ -627,7 +703,7 @@ LANGUAGE sql;
 
 -- get_version() - version of statsrepo schema
 CREATE FUNCTION statsrepo.get_version() RETURNS text AS
-'SELECT CAST(''30300'' AS TEXT)'
+'SELECT CAST(''100000'' AS TEXT)'
 LANGUAGE sql IMMUTABLE;
 
 -- tps() - transaction per seconds
@@ -2458,19 +2534,23 @@ CREATE FUNCTION statsrepo.get_query_activity_statements(
 $$
 DECLARE
 	pg_version_num	integer;
+	vmaj			integer;
+	vmin			integer;
+	vrev			integer;
 BEGIN
 	SELECT
-		CASE WHEN split_part(pg_version, '.', 3) != '' THEN
-			(split_part(pg_version, '.', 1)::integer * 100 +
-			 split_part(pg_version, '.', 2)::integer) * 100 +
-			 split_part(pg_version, '.', 3)::integer
-		ELSE
-			(split_part(pg_version, '.', 1)::integer * 100 +
-			 substring(split_part(pg_version, '.', 2) from '^\d+')::integer) * 100
-		END
-	INTO pg_version_num FROM statsrepo.instance
+		coalesce(substring(split_part(pg_version, '.', 1) from '^\d+')::integer, 0),
+		coalesce(substring(split_part(pg_version, '.', 2) from '^\d+')::integer, 0),
+		coalesce(substring(split_part(pg_version, '.', 3) from '^\d+')::integer, 0)
+	INTO vmaj, vmin, vrev FROM statsrepo.instance
 	WHERE instid = (SELECT instid FROM statsrepo.snapshot WHERE snapid = $1);
 
+	IF vmaj >= 10 THEN
+		pg_version_num := 100 * 100 * vmaj + vmin;
+	ELSE
+		pg_version_num := (100 * vmaj + vmin) * 100 + vrev;
+	END IF;
+
 	IF pg_version_num >= 90400 THEN
 		RETURN QUERY SELECT * FROM statsrepo.get_query_activity_statemets_body($1, $2);
 	ELSE
@@ -2951,7 +3031,10 @@ LANGUAGE sql;
 CREATE FUNCTION statsrepo.get_replication_delays(
 	IN snapid_begin			bigint,
 	IN snapid_end			bigint,
+	OUT snapid				bigint,
 	OUT "timestamp"			text,
+	OUT client_addr			inet,
+	OUT application_name	text,
 	OUT client				text,
 	OUT flush_delay_size	numeric,
 	OUT replay_delay_size	numeric,
@@ -2959,8 +3042,11 @@ CREATE FUNCTION statsrepo.get_replication_delays(
 ) RETURNS SETOF record AS
 $$
 	SELECT
+		s.snapid,
 		to_char(s.time, 'YYYY-MM-DD HH24:MI'),
-		host(r.client_addr) || ':' || r.client_port AS client,
+		r.client_addr,
+		r.application_name,
+		COALESCE(host(r.client_addr), 'local') || ':' || COALESCE(NULLIF(r.application_name, ''), '(none)') AS client,
 		statsrepo.xlog_location_diff(
 			split_part(r.current_location, ' ', 1),
 			split_part(r.flush_location, ' ', 1),
@@ -2971,7 +3057,8 @@ $$
 			i.xlog_file_size),
 		r.sync_state
 	FROM
-		statsrepo.replication r,
+		statsrepo.replication r LEFT JOIN statsrepo.replication_slots rs
+			ON rs.snapid = r.snapid AND rs.active_pid = r.procpid,
 		statsrepo.snapshot s,
 		statsrepo.instance i
 	WHERE
@@ -2981,6 +3068,7 @@ $$
 		AND s.instid = (SELECT instid FROM statsrepo.snapshot WHERE snapid = $2)
 		AND r.flush_location IS NOT NULL
 		AND r.replay_location IS NOT NULL
+		AND (rs.slot_name IS NULL OR rs.slot_type = 'physical')
 	ORDER BY
 		s.snapid, client;
 $$
@@ -2990,6 +3078,7 @@ LANGUAGE sql;
 CREATE FUNCTION statsrepo.get_replication_activity(
 	IN snapid_begin			bigint,
 	IN snapid_end			bigint,
+	OUT snapshot_time		timestamp,
 	OUT usename				name,
 	OUT application_name	text,
 	OUT client_addr			inet,
@@ -3005,10 +3094,15 @@ CREATE FUNCTION statsrepo.get_replication_activity(
 	OUT sync_priority		integer,
 	OUT sync_state			text,
 	OUT replay_delay_avg	numeric,
-	OUT replay_delay_peak	numeric
+	OUT replay_delay_peak	numeric,
+	OUT write_lag_time		interval,
+	OUT flush_lag_time		interval,
+	OUT replay_lag_time		interval,
+	OUT priority_sortkey	integer
 ) RETURNS SETOF record AS
 $$
 	SELECT
+		s.time::timestamp(0),
 		r.usename,
 		r.application_name,
 		r.client_addr,
@@ -3024,20 +3118,28 @@ $$
 		r.sync_priority,
 		r.sync_state,
 		d.replay_delay_avg,
-		d.replay_delay_peak
+		d.replay_delay_peak,
+		r.write_lag,
+		r.flush_lag,
+		r.replay_lag,
+		NULLIF(r.sync_priority, 0) AS priority_sortkey
 	FROM
-		statsrepo.replication r,
 		(SELECT
-			client,
+			client_addr,
+			application_name,
+			max(snapid) AS snapid,
 			avg(replay_delay_size) AS replay_delay_avg,
 			max(replay_delay_size) AS replay_delay_peak
 		 FROM
-		 	statsrepo.get_replication_delays($1, $2)
+			statsrepo.get_replication_delays($1, $2)
 		 GROUP BY
-			client) d
-	WHERE
-		d.client = (host(r.client_addr) || ':' || r.client_port)
-		AND r.snapid = $2;
+			client_addr, application_name) d
+		LEFT JOIN statsrepo.replication r
+			ON r.snapid = d.snapid
+			AND d.client_addr = r.client_addr
+			AND d.application_name = r.application_name
+		LEFT JOIN statsrepo.snapshot s ON s.snapid = d.snapid
+	ORDER BY d.snapid DESC, priority_sortkey ASC NULLS LAST;
 $$
 LANGUAGE sql;
 
